@@ -163,14 +163,18 @@ class Consumer(object):
             if message is StopIteration:
                 break
 
-            self.debug("Worker thread picking up %r" % message)
+            self.debug("Worker thread picking a message.")
             try:
                 self.validate(message)
             except Exception as e:
                 log.warn("Received invalid message %r" % e)
                 continue
 
-            self.consume(message)
+            try:
+                self.consume(message)
+            except Exception as e:
+                self.log.exception(message)
+
             self.debug("Going back to waiting on the incoming queue.")
 
         self.debug("Worker thread exiting.")
