@@ -1,5 +1,5 @@
 # This file is part of Moksha.
-# Copyright (C) 2008-2010  Red Hat, Inc.
+# Copyright (C) 2008-2014  Red Hat, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@
 
 
 import os
+import six
 import sys
 import json as JSON
 from collections import defaultdict
@@ -31,12 +32,6 @@ if os.getcwd() not in sys.path:
 import pkg_resources
 import logging
 
-try:
-    from twisted.internet.error import ReactorNotRunning
-except ImportError:  # Twisted 8.2.0 on RHEL5
-    class ReactorNotRunning(object):
-        pass
-
 from twisted.internet import protocol
 from txws import WebSocketFactory
 from moksha.common.lib.helpers import get_moksha_config_path
@@ -71,6 +66,7 @@ def find_hub_extensions(config):
     possible_bases = {
         'amqp_broker': AMQPHubExtension,
         'stomp_broker': StompHubExtension,
+        'stomp_uri': StompHubExtension,
         'zmq_enabled': ZMQHubExtension,
     }
 
@@ -90,12 +86,12 @@ def find_hub_extensions(config):
     if not any(broker_vals):
         raise ValueError("No messaging methods defined.")
 
-    if len(filter(None, broker_vals)) > 1:
+    if len(list(filter(None, broker_vals))) > 1:
         log.warning("Running with multiple brokers.  "
                     "This mode is experimental and may or may not work")
 
-    extensions = tuple([
-        b for k, b in possible_bases.items() if config.get(k, None)
+    extensions = set([
+        b for k, b in possible_bases.items() if config.get(k, None) and b
     ])
     return extensions
 
@@ -112,7 +108,7 @@ class MokshaHub(object):
         if topics is None:
             topics = {}
 
-        for topic, callbacks in topics.iteritems():
+        for topic, callbacks in topics.items():
             if not isinstance(callbacks, list):
                 callbacks = [callbacks]
 
@@ -141,7 +137,7 @@ class MokshaHub(object):
             topics = topic
 
         for topic in topics:
-            if isinstance(topic, unicode):
+            if isinstance(topic, six.text_type):
                 # txzmq isn't smart enough to handle unicode yet.
                 # Try removing this and sending a unicode topic in the future
                 # to see if it works.
@@ -155,8 +151,8 @@ class MokshaHub(object):
             for ext in self.extensions:
                 if hasattr(ext, 'close'):
                     ext.close()
-        except Exception, e:
-            log.warning('Exception when closing MokshaHub: %s' % str(e))
+        except Exception as e:
+            log.warning('Exception when closing MokshaHub: %r' % e)
 
     def unsubscribe(self, callback):
         """
@@ -200,8 +196,8 @@ class MokshaHub(object):
 
         # FIXME: only do this if the consumer wants it `jsonified`
         try:
-            body = JSON.decode(message['body'])
-        except Exception, e:
+            body = JSON.loads(message['body'])
+        except Exception as e:
             log.warning('Cannot decode message from JSON: %s' % e)
             #body = {}
             body = message['body']
@@ -276,6 +272,7 @@ class CentralMokshaHub(MokshaHub):
                 """
 
                 try:
+                    data = data.decode('utf-8')
                     json = JSON.loads(data)
 
                     if json['topic'] == '__topic_subscribe__':
@@ -395,6 +392,11 @@ class CentralMokshaHub(MokshaHub):
             log.info('Initializing %s producer' % producer_class.__name__)
             try:
                 producer_obj = producer_class(self)
+
+                if not getattr(producer_obj, "_initialized", None):
+                    log.info("%s:%s not initialized." % (
+                        producer_class.__module__, producer_class.__name__,))
+
                 self.producers.append(producer_obj)
             except Exception as e:
                 log.exception("Failed to init %r producer." % producer_class)
@@ -409,6 +411,7 @@ class CentralMokshaHub(MokshaHub):
 
     def close(self):
         log.debug("Stopping the CentralMokshaHub")
+
         super(CentralMokshaHub, self).close()
 
         if self.producers:
@@ -423,9 +426,6 @@ class CentralMokshaHub(MokshaHub):
                 log.debug("Stopping consumer %s" % consumer)
                 consumer.stop()
 
-        if hasattr(self, 'websocket_server'):
-            retval = self.websocket_server.stopListening()
-
     # For backwards compatibility
     stop = close
 
